==> arirang-2.02.tar.gz: arirang-2.02/proxy.c <==
/*
 * Copyright 2010, 2011 pilot <pilot@monkey.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>
#include <resolv.h>

#include "arirang.h"


#define SOCKS5_VER 0x05
#define SOCKS5_NMETHODS 0x01
#define SOCKS5_NOAUTH_METHOD 0x00
#define SOCKS5_CMD_CONNECT 0x01
#define SOCKS5_RSV 0x00
#define SOCKS5_ATYP 0x03 /* IPv4 is 0x01 */

extern short int CONNECT_TIMEOUT;
extern short int RECV_TIMEOUT;

enum SOCKS5_REP
{
    SOCKS5_REP_SUCCEEDED,
    SOCKS5_REP_GEN_FAIL,
    SOCKS5_REP_CON_NOT,
    SOCKS5_REP_NET_UNREACH,
    SOCKS5_REP_HOST_UNREACH,
    SOCKS5_REP_CON_REFUSE,
    SOCKS5_REP_TTL_EXPIRE,
    SOCKS5_REP_CMD_NOT,
    SOCKS5_REP_ADDR_NOT,
    SOCKS5_REP_INVALID
};

int 
proxy_connect(char *scanhost, unsigned int port, char *proxyhost, 
    unsigned int proxyport, int type)
{
    char sendbuf[256];
    char headbuf[1024];

    char auth_user[32]; 
    char auth_pw[32];
    char proxy_type[32];
    char auth_buf[72];
    char auth_b64[128];
    char *env_user = NULL;
    char *env_pw = NULL;
    char *env_type = NULL;
    int env_auth = 0;
    int proxy_enum = 0;

    int base64 = 0;
    int socks5_len = 0;
    enum SOCKS5_REP socks5_rep;

    int sock;
    int rt;

    struct sockaddr_in http;
    struct hostent *he;

    if ((he = gethostbyname(proxyhost)) == NULL) {
        perror("proxy host not found");
        close(sock);  
        exit(0);
    }


    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
        perror("socket");
        close(sock);
        exit(0);
    }
    http.sin_family = AF_INET;
    http.sin_port = htons(proxyport);
    http.sin_addr = *((struct in_addr *) he->h_addr);
    bzero(&(http.sin_zero), 8);

    signal(SIGALRM, connout);
    alarm(CONNECT_TIMEOUT);
    
    if (connect(sock, (struct sockaddr *) & http, sizeof(struct sockaddr)) == -1) {
        fprintf(stderr, "%s:%d - proxy server could not connect\n", proxyhost, proxyport);
        close(sock);
        exit(0);
    }
    alarm(0);

    if ((env_user = getenv("PROXY_USER")) != NULL) {
        env_auth = 1;
        snprintf(auth_user, sizeof(auth_user), "%s", env_user);
    }

    if ((env_pw = getenv("PROXY_PASS")) != NULL) {
        env_auth = 1;
        snprintf(auth_pw, sizeof(auth_pw), "%s", env_pw);
    }

    if ((env_type = getenv("PROXY_TYPE")) != NULL) {
        snprintf(proxy_type, sizeof(proxy_type), "%s", env_type);
    }

    if (strstr(proxy_type, "HTTP") != NULL)
        proxy_enum = 0;

/*  Caching(Relay) only HTTP requests such as Delegate and other
    only use with arirang script.
*/
    if (strstr(proxy_type, "RELAY") != NULL) {
        proxy_enum = 1;
        return sock;
    }

    if (strstr(proxy_type, "SOCKS5") != NULL)
        proxy_enum = 5;
   
#ifdef DEBUG
    printf("auth_user env = %s\n", auth_user);
    printf("auth_pass env = %s\n", auth_pw);
    printf("proxy_type env = %s\n", proxy_type);
#endif

/* HTTP Proxy */
  if (proxy_enum == 0) {
    if (!env_auth)     
        snprintf(sendbuf, sizeof(sendbuf), "CONNECT %s:%d HTTP/1.0\r\n\r\n", scanhost, port);

    if (env_auth) { 
        snprintf(auth_buf, sizeof(auth_buf), "%s:%s", auth_user, auth_pw);
        base64 = b64_ntop((unsigned char *)auth_buf, strlen(auth_buf), auth_b64, sizeof(auth_b64));
#ifdef DEBUG
        printf("base64 length = %d, base64 encode = %s\n", base64, auth_b64);
#endif
        if (base64 < 0)
            fprintf(stderr, "base64 data length less than zero\n");

        snprintf(sendbuf, sizeof(sendbuf), "CONNECT %s:%d HTTP/1.0\r\nProxy-Authorization: Basic %s\r\n\r\n", 
            scanhost, port, auth_b64);
    }

    send(sock, sendbuf, strlen(sendbuf), 0);
    rt = recvtimeout(sock, headbuf, sizeof(headbuf), RECV_TIMEOUT);

    if ((strstr(headbuf, "HTTP/1.0 407") != NULL) && env_auth) {
        fprintf(stderr, "%s:%d - http proxy server auth failed \n", proxyhost, proxyport);
        close(sock);
        exit(0);
    }

    if ((strstr(headbuf, "HTTP/1.0 200") != NULL) || (strstr(headbuf, "HTTP/1.1 200") != NULL)) {
        return sock;
    }

  }

/* SOCKS5 Proxy */
  if (proxy_enum == 5) {
    sendbuf[0] = SOCKS5_VER;
    sendbuf[1] = SOCKS5_NMETHODS;
    sendbuf[2] = SOCKS5_NOAUTH_METHOD;

    send(sock, sendbuf, 3, 0);
    rt = recvtimeout(sock, headbuf, sizeof(headbuf), RECV_TIMEOUT);

    socks5_rep = headbuf[1];

/* REP    Reply field:
             o  X'00' succeeded
             o  X'01' general SOCKS server failure
             o  X'02' connection not allowed by ruleset
             o  X'03' Network unreachable
             o  X'04' Host unreachable
             o  X'05' Connection refused
             o  X'06' TTL expired
             o  X'07' Command not supported
             o  X'08' Address type not supported
             o  X'09' to X'FF' unassigned
*/

    switch (socks5_rep) {
    case SOCKS5_REP_SUCCEEDED:
#ifdef DEBUG
        fprintf(stdout, "%s:%d - succeeded \n", proxyhost, proxyport);
#endif
        break;
    case SOCKS5_REP_GEN_FAIL:
        fprintf(stderr, "%s:%d - general SOCKS server failure \n", proxyhost, proxyport);
    case SOCKS5_REP_CON_NOT:
        fprintf(stderr, "%s:%d - connection not allowed by ruleset \n", proxyhost, proxyport);
    case SOCKS5_REP_NET_UNREACH:
        fprintf(stderr, "%s:%d - Network unreachable \n", proxyhost, proxyport);
    case SOCKS5_REP_HOST_UNREACH:
        fprintf(stderr, "%s:%d - Host unreachable \n", proxyhost, proxyport);
    case SOCKS5_REP_CON_REFUSE:
        fprintf(stderr, "%s:%d - Connection refused \n", proxyhost, proxyport);
    case SOCKS5_REP_TTL_EXPIRE:
        fprintf(stderr, "%s:%d - TTL expired \n", proxyhost, proxyport);
    case SOCKS5_REP_CMD_NOT:
        fprintf(stderr, "%s:%d - Command not supported \n", proxyhost, proxyport);
    case SOCKS5_REP_ADDR_NOT:
        fprintf(stderr, "%s:%d - Address type not supported \n", proxyhost, proxyport);
    case SOCKS5_REP_INVALID:
        fprintf(stderr, "%s:%d - unassigned \n", proxyhost, proxyport);
    default:
        close(sock);
        exit(0);
    }

/*
        +----+-----+-------+------+----------+----------+
        |VER | CMD |  RSV  | ATYP | DST.ADDR | DST.PORT |
        +----+-----+-------+------+----------+----------+
        | 1  |  1  | X'00' |  1   | Variable |    2     |
        +----+-----+-------+------+----------+----------+
*/ 
    sendbuf[0] = SOCKS5_VER;
    sendbuf[1] = SOCKS5_CMD_CONNECT;
    sendbuf[2] = SOCKS5_RSV;
    sendbuf[3] = SOCKS5_ATYP;
    sendbuf[4] = strlen(scanhost);
    socks5_len = socks5_len + 5;
    memcpy(sendbuf + 5, scanhost, strlen(scanhost));
    socks5_len = socks5_len + strlen(scanhost);
    sendbuf[socks5_len] = (char)((port >> 8) & 0xff);
    sendbuf[socks5_len+1] = (char)(port & 0xff);

    send(sock, sendbuf, socks5_len + 2, 0);
    rt = recvtimeout(sock, headbuf, sizeof(headbuf), RECV_TIMEOUT);

  }

    return sock;
}

==> arirang-2.03.tar.gz: arirang-2.03/proxy.c <==
/*
 * Copyright 2010, 2011 pilot <pilot@monkey.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>
#include <resolv.h>

#include "arirang.h"


#define SOCKS5_VER 0x05
#define SOCKS5_NMETHODS 0x01
#define SOCKS5_NOAUTH_METHOD 0x00
#define SOCKS5_CMD_CONNECT 0x01
#define SOCKS5_RSV 0x00
#define SOCKS5_ATYP 0x03 /* IPv4 is 0x01 */

extern short int CONNECT_TIMEOUT;
extern short int RECV_TIMEOUT;

enum SOCKS5_REP
{
    SOCKS5_REP_SUCCEEDED,
    SOCKS5_REP_GEN_FAIL,
    SOCKS5_REP_CON_NOT,
    SOCKS5_REP_NET_UNREACH,
    SOCKS5_REP_HOST_UNREACH,
    SOCKS5_REP_CON_REFUSE,
    SOCKS5_REP_TTL_EXPIRE,
    SOCKS5_REP_CMD_NOT,
    SOCKS5_REP_ADDR_NOT,
    SOCKS5_REP_INVALID
};

int 
proxy_connect(char *scanhost, unsigned int port, char *proxyhost, 
    unsigned int proxyport, int type)
{
    char sendbuf[256];
    char headbuf[1024];

    char auth_user[32]; 
    char auth_pw[32];
    char proxy_type[32];
    char auth_buf[72];
    char auth_b64[128];
    char *env_user = NULL;
    char *env_pw = NULL;
    char *env_type = NULL;
    int env_auth = 0;
    int proxy_enum = 0;

    int base64 = 0;
    int socks5_len = 0;
    enum SOCKS5_REP socks5_rep;

    int sock;
    int rt;

    struct sockaddr_in http;
    struct hostent *he;

    if ((he = gethostbyname(proxyhost)) == NULL) {
        perror("proxy host not found");
        close(sock);  
        exit(0);
    }


    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
        perror("socket");
        close(sock);
        exit(0);
    }
    http.sin_family = AF_INET;
    http.sin_port = htons(proxyport);
    http.sin_addr = *((struct in_addr *) he->h_addr);
    bzero(&(http.sin_zero), 8);

    signal(SIGALRM, connout);
    alarm(CONNECT_TIMEOUT);
    
    if (connect(sock, (struct sockaddr *) & http, sizeof(struct sockaddr)) == -1) {
        fprintf(stderr, "%s:%d - proxy server could not connect\n", proxyhost, proxyport);
        close(sock);
        exit(0);
    }
    alarm(0);

    if ((env_user = getenv("PROXY_USER")) != NULL) {
        env_auth = 1;
        snprintf(auth_user, sizeof(auth_user), "%s", env_user);
    }

    if ((env_pw = getenv("PROXY_PASS")) != NULL) {
        env_auth = 1;
        snprintf(auth_pw, sizeof(auth_pw), "%s", env_pw);
    }

    if ((env_type = getenv("PROXY_TYPE")) != NULL) {
        snprintf(proxy_type, sizeof(proxy_type), "%s", env_type);
    }

    if (strstr(proxy_type, "HTTP") != NULL)
        proxy_enum = 0;

/*  Caching(Relay) only HTTP requests such as Delegate and other
    only use with arirang script.
*/
    if (strstr(proxy_type, "RELAY") != NULL) {
        proxy_enum = 1;
        return sock;
    }

    if (strstr(proxy_type, "SOCKS5") != NULL)
        proxy_enum = 5;
   
    DPRINTF("[info] proxy auth_user env - %s\n", auth_user);
    DPRINTF("[info] proxy auth_pass env - %s\n", auth_pw);
    DPRINTF("[info] proxy type - %s\n", proxy_type);

/* HTTP Proxy */
  if (proxy_enum == 0) {
    if (!env_auth)     
        snprintf(sendbuf, sizeof(sendbuf), "CONNECT %s:%d HTTP/1.0\r\n\r\n", scanhost, port);

    if (env_auth) { 
        snprintf(auth_buf, sizeof(auth_buf), "%s:%s", auth_user, auth_pw);
        base64 = b64_ntop((unsigned char *)auth_buf, strlen(auth_buf), auth_b64, sizeof(auth_b64));
        
		DPRINTF("[info] proxy base64 length = %d, base64 encode = %s\n", base64, auth_b64);

        if (base64 < 0)
            fprintf(stderr, "base64 data length less than zero\n");

        snprintf(sendbuf, sizeof(sendbuf), "CONNECT %s:%d HTTP/1.0\r\nProxy-Authorization: Basic %s\r\n\r\n", 
            scanhost, port, auth_b64);
    }

    send(sock, sendbuf, strlen(sendbuf), 0);
    rt = recvtimeout(sock, headbuf, sizeof(headbuf), RECV_TIMEOUT);

    if ((strstr(headbuf, "HTTP/1.0 407") != NULL) && env_auth) {
        fprintf(stderr, "%s:%d - http proxy server auth failed \n", proxyhost, proxyport);
        close(sock);
        exit(0);
    }

    if ((strstr(headbuf, "HTTP/1.0 200") != NULL) || (strstr(headbuf, "HTTP/1.1 200") != NULL)) {
        return sock;
    }

  }

/* SOCKS5 Proxy */
  if (proxy_enum == 5) {
    sendbuf[0] = SOCKS5_VER;
    sendbuf[1] = SOCKS5_NMETHODS;
    sendbuf[2] = SOCKS5_NOAUTH_METHOD;

    send(sock, sendbuf, 3, 0);
    rt = recvtimeout(sock, headbuf, sizeof(headbuf), RECV_TIMEOUT);

    socks5_rep = headbuf[1];

/* REP    Reply field:
             o  X'00' succeeded
             o  X'01' general SOCKS server failure
             o  X'02' connection not allowed by ruleset
             o  X'03' Network unreachable
             o  X'04' Host unreachable
             o  X'05' Connection refused
             o  X'06' TTL expired
             o  X'07' Command not supported
             o  X'08' Address type not supported
             o  X'09' to X'FF' unassigned
*/

    switch (socks5_rep) {
    case SOCKS5_REP_SUCCEEDED:
        DPRINTF("[info] socks5 %s:%d - succeeded \n", proxyhost, proxyport);
        break;
    case SOCKS5_REP_GEN_FAIL:
        fprintf(stderr, "%s:%d - general SOCKS server failure \n", proxyhost, proxyport);
    case SOCKS5_REP_CON_NOT:
        fprintf(stderr, "%s:%d - connection not allowed by ruleset \n", proxyhost, proxyport);
    case SOCKS5_REP_NET_UNREACH:
        fprintf(stderr, "%s:%d - Network unreachable \n", proxyhost, proxyport);
    case SOCKS5_REP_HOST_UNREACH:
        fprintf(stderr, "%s:%d - Host unreachable \n", proxyhost, proxyport);
    case SOCKS5_REP_CON_REFUSE:
        fprintf(stderr, "%s:%d - Connection refused \n", proxyhost, proxyport);
    case SOCKS5_REP_TTL_EXPIRE:
        fprintf(stderr, "%s:%d - TTL expired \n", proxyhost, proxyport);
    case SOCKS5_REP_CMD_NOT:
        fprintf(stderr, "%s:%d - Command not supported \n", proxyhost, proxyport);
    case SOCKS5_REP_ADDR_NOT:
        fprintf(stderr, "%s:%d - Address type not supported \n", proxyhost, proxyport);
    case SOCKS5_REP_INVALID:
        fprintf(stderr, "%s:%d - unassigned \n", proxyhost, proxyport);
    default:
        close(sock);
        exit(0);
    }

/*
        +----+-----+-------+------+----------+----------+
        |VER | CMD |  RSV  | ATYP | DST.ADDR | DST.PORT |
        +----+-----+-------+------+----------+----------+
        | 1  |  1  | X'00' |  1   | Variable |    2     |
        +----+-----+-------+------+----------+----------+
*/ 
    sendbuf[0] = SOCKS5_VER;
    sendbuf[1] = SOCKS5_CMD_CONNECT;
    sendbuf[2] = SOCKS5_RSV;
    sendbuf[3] = SOCKS5_ATYP;
    sendbuf[4] = strlen(scanhost);
    socks5_len = socks5_len + 5;
    memcpy(sendbuf + 5, scanhost, strlen(scanhost));
    socks5_len = socks5_len + strlen(scanhost);
    sendbuf[socks5_len] = (char)((port >> 8) & 0xff);
    sendbuf[socks5_len+1] = (char)(port & 0xff);

    send(sock, sendbuf, socks5_len + 2, 0);
    rt = recvtimeout(sock, headbuf, sizeof(headbuf), RECV_TIMEOUT);

  }

    return sock;
}
